@@ -2,7 +2,7 @@
 """
 :Author: HuangJingCan
 :Date: 2020-05-19 11:33:16
-@LastEditTime: 2021-07-26 19:52:38
+@LastEditTime: 2021-08-31 17:09:48
 @LastEditors: HuangJianYi
 :description: 用户处理
 """
@@ -11,6 +11,7 @@ from seven_cloudapp.handlers.top_base import *
 from seven_cloudapp.models.behavior_model import *
 from seven_cloudapp.models.seven_model import PageInfo
 from seven_cloudapp.models.db_models.user.user_info_model import *
+from seven_cloudapp.models.db_models.user.user_detail_model import *
 from seven_cloudapp.models.db_models.app.app_info_model import *
 from seven_cloudapp.models.db_models.pay.pay_order_model import *
 from seven_cloudapp.models.db_models.prize.prize_order_model import *
@@ -18,6 +19,7 @@ from seven_cloudapp.models.db_models.coin.coin_order_model import *
 from seven_cloudapp.models.db_models.price.price_gear_model import *
 from seven_cloudapp.models.db_models.gear.gear_value_model import *
 from seven_cloudapp.models.db_models.ip.ip_series_model import *
+from seven_cloudapp.models.db_models.prop.prop_log_model import *
 
 from seven_cloudapp.handlers.client.user import UserHandler
 from seven_cloudapp.handlers.client.user import LoginHandler
@@ -28,6 +30,7 @@ from seven_cloudapp_ppmt.models.db_models.act.act_prize_model import *
 from seven_cloudapp_ppmt.models.db_models.machine.machine_info_model import *
 from seven_cloudapp_ppmt.models.db_models.prize.prize_roster_model import *
 from seven_cloudapp_ppmt.models.db_models.act.act_info_model import *
+from seven_cloudapp_ppmt.models.db_models.prop.prop_give_model import *
 
 
 class SyncPayOrderHandler(TopBaseHandler):
@@ -67,10 +70,6 @@ class SyncPayOrderHandler(TopBaseHandler):
         order_data = self.get_taobao_order(open_id, access_token)
         # self.logging_link_info(str(order_data) + "【订单列表】")
 
-        #获取所有盲盒配置
-        machine_info_model = MachineInfoModel(context=self)
-        machine_config_list = machine_info_model.get_list("act_id=%s", params=[act_id])
-
         #获取价格档位配置
         price_gear_model = PriceGearModel(context=self)
         price_gear_list = price_gear_model.get_list("act_id=%s", params=[act_id])
@@ -504,4 +503,224 @@ class GetHorseRaceLampListHandler(SevenBaseHandler):
                         prize_roster["machine_name"] = machine_info_filter[0].machine_name if len(machine_info_filter) > 0 else ""
                         page_list.append(prize_roster)
 
-        return self.reponse_json_success(page_list)
\ No newline at end of file
+        return self.reponse_json_success(page_list)
+
+
+class GivePropHandler(SevenBaseHandler):
+    """
+    :description:道具卡赠送
+    """
+    @filter_check_params("act_id,login_token,prop_type")
+    def get_async(self):
+        """
+        :description: 道具卡赠送
+        :param act_id:活动id
+        :param login_token:登录令牌
+        :param prop_type:道具类型(2透视卡3提示卡4重抽卡)
+        :return:
+        :last_editors: HuangJianYi
+        """
+        app_id = self.get_taobao_param().source_app_id
+        open_id = self.get_taobao_param().open_id
+        act_id = int(self.get_param("act_id", 0))
+        prop_type = int(self.get_param("prop_type", 2))
+        login_token = self.get_param("login_token")
+
+        db_transaction = DbTransaction(db_config_dict=config.get_value("db_cloudapp"))
+        act_info_model = ActInfoModel(context=self)
+        user_info_model = UserInfoModel(context=self)
+        user_detail_model = UserDetailModel(db_transaction=db_transaction, context=self)
+        prop_log_model = PropLogModel(db_transaction=db_transaction, context=self)
+        prop_give_model = PropGiveModel(db_transaction=db_transaction, context=self)
+        #请求太频繁限制
+        if self.check_post(f"PropGive_Post_{str(act_id)}_{str(open_id)}") == False:
+            return self.reponse_json_error("HintMessage", "对不起，请求太频繁")
+        act_dict = act_info_model.get_dict("id=%s and is_release=1", params=act_id)
+        if not act_dict:
+            return self.reponse_json_error("NoAct", "对不起，活动不存在")
+
+        now_date = self.get_now_datetime()
+        if act_dict['start_date'] != "":
+            if TimeHelper.format_time_to_datetime(now_date) < TimeHelper.format_time_to_datetime(act_dict['start_date']):
+                return self.reponse_json_error("NoAct", "活动将在" + str(act_dict['start_date']) + "开启")
+        if act_dict['end_date'] != "":
+            if TimeHelper.format_time_to_datetime(now_date) > TimeHelper.format_time_to_datetime(act_dict['end_date']):
+                return self.reponse_json_error("NoAct", "活动已结束")
+        user_info = user_info_model.get_entity("open_id=%s and act_id=%s", params=[open_id, act_id])
+        if not user_info or user_info.act_id != act_id:
+            return self.reponse_json_error("Error", "对不起，用户不存在")
+        if user_info.user_state == 1:
+            return self.reponse_json_error("UserState", "账号异常，请联系客服处理")
+        if user_info.login_token != login_token:
+            return self.reponse_json_error("Error", "对不起，已在另一台设备登录,当前无法操作")
+        user_detail = user_detail_model.get_entity("open_id=%s and act_id=%s", params=[open_id, act_id])
+        history_value = 0
+        update_sql = ""
+        if not user_detail:
+            return self.reponse_json_error("Error", "道具卡数量不足")
+        give_config_json = self.json_loads(act_dict["give_config_json"])
+        if not give_config_json:
+            return self.reponse_json_error("Error", "无法进行赠送")
+
+        prop_give_total = prop_give_model.get_total("open_id=%s and act_id=%s and prop_type=%s and create_date_int=%s", params=[open_id, act_id, prop_type, SevenHelper.get_now_day_int()])
+        if prop_type == 3:
+            if user_detail.tips_card_count <= 0:
+                return self.reponse_json_error("Error", "提示卡数量不足")
+            if prop_give_total >= int(give_config_json["tips_limit_count"]):
+                return self.reponse_json_error("Error", "提示卡赠送数量达到上限")
+            history_value = user_detail.tips_card_count
+            update_sql = "tips_card_count=tips_card_count-1"
+        elif prop_type == 4:
+            if user_detail.redraw_card_count <= 0:
+                return self.reponse_json_error("Error", "重抽卡数量不足")
+            if prop_give_total >= int(give_config_json["redraw_limit_count"]):
+                return self.reponse_json_error("Error", "重抽卡赠送数量达到上限")
+            history_value = user_detail.redraw_card_count
+            update_sql = "redraw_card_count=redraw_card_count-1"
+        else:
+            if user_detail.perspective_card_count <= 0:
+                return self.reponse_json_error("Error", "透视卡数量不足")
+            if prop_give_total >= int(give_config_json["perspective_limit_count"]):
+                return self.reponse_json_error("Error", "透视卡赠送数量达到上限")
+            history_value = user_detail.perspective_card_count
+            update_sql = "perspective_card_count=perspective_card_count-1"
+
+        prop_log = PropLog()
+        prop_log.app_id = app_id
+        prop_log.act_id = act_id
+        prop_log.open_id = open_id
+        prop_log.user_nick = user_info.user_nick
+        prop_log.change_type = 4
+        prop_log.operate_type = 1
+        prop_log.prop_type = prop_type
+        prop_log.operate_value = -1
+        prop_log.history_value = history_value
+        prop_log.title = "道具赠送"
+        prop_log.create_date_int = SevenHelper.get_now_day_int()
+        prop_log.create_date = self.get_now_datetime()
+        info = {}
+        prop_log.remark = info
+
+        guid = UUIDHelper.get_uuid()
+        prop_give = PropGive()
+        prop_give.guid = guid
+        prop_give.app_id = app_id
+        prop_give.act_id = act_id
+        prop_give.open_id = open_id
+        prop_give.user_nick = user_info.user_nick
+        prop_give.prop_type = prop_type
+        prop_give.give_num = 1
+        prop_give.give_status = 0
+        prop_give.create_date_int = SevenHelper.get_now_day_int()
+        prop_give.create_date = self.get_now_datetime()
+
+
+        try:
+            db_transaction.begin_transaction()
+            prop_log_model.add_entity(prop_log)
+            prop_give_model.add_entity(prop_give)
+            user_detail_model.update_table(update_sql, "open_id=%s and act_id=%s", params=[open_id, act_id])
+            db_transaction.commit_transaction()
+
+        except Exception as ex:
+            db_transaction.rollback_transaction()
+            self.logging_link_error("GivePropHandler:" + str(ex))
+            return self.reponse_json_error("Error", "系统繁忙,请稍后再试")
+        return self.reponse_json_success(guid)
+
+
+class GetPropGiveHandler(SevenBaseHandler):
+    """
+    :description:领取赠送的道具卡
+    """
+    @filter_check_params("act_id,login_token,give_guid")
+    def get_async(self):
+        """
+        :description: 道具卡赠送
+        :param act_id:活动id
+        :param login_token:登录令牌
+        :param give_guid:赠送guid
+        :return:
+        :last_editors: HuangJianYi
+        """
+        app_id = self.get_taobao_param().source_app_id
+        open_id = self.get_taobao_param().open_id
+        act_id = int(self.get_param("act_id", 0))
+        give_guid = self.get_param("give_guid")
+        login_token = self.get_param("login_token")
+
+        db_transaction = DbTransaction(db_config_dict=config.get_value("db_cloudapp"))
+        act_info_model = ActInfoModel(context=self)
+        user_info_model = UserInfoModel(context=self)
+        user_detail_model = UserDetailModel(db_transaction=db_transaction, context=self)
+        prop_log_model = PropLogModel(db_transaction=db_transaction, context=self)
+        prop_give_model = PropGiveModel(db_transaction=db_transaction, context=self)
+        #请求太频繁限制
+        if self.check_post(f"PropGive_Post_{str(act_id)}_{str(open_id)}") == False:
+            return self.reponse_json_error("HintMessage", "对不起，请求太频繁")
+        act_dict = act_info_model.get_dict("id=%s and is_release=1", params=act_id)
+        if not act_dict:
+            return self.reponse_json_error("NoAct", "对不起，活动不存在")
+        now_date = self.get_now_datetime()
+        if act_dict['start_date'] != "":
+            if TimeHelper.format_time_to_datetime(now_date) < TimeHelper.format_time_to_datetime(act_dict['start_date']):
+                return self.reponse_json_error("NoAct", "活动将在" + str(act_dict['start_date']) + "开启")
+        if act_dict['end_date'] != "":
+            if TimeHelper.format_time_to_datetime(now_date) > TimeHelper.format_time_to_datetime(act_dict['end_date']):
+                return self.reponse_json_error("NoAct", "活动已结束")
+        user_info = user_info_model.get_entity("open_id=%s and act_id=%s", params=[open_id, act_id])
+        if not user_info or user_info.act_id != act_id:
+            return self.reponse_json_error("Error", "对不起，用户不存在")
+        if user_info.user_state == 1:
+            return self.reponse_json_error("UserState", "账号异常，请联系客服处理")
+        if user_info.login_token != login_token:
+            return self.reponse_json_error("Error", "对不起，已在另一台设备登录,当前无法操作")
+
+        prop_give = prop_give_model.get_entity("guid=%s", params=[give_guid])
+        if not prop_give:
+            return self.reponse_json_error("Error", "不存在赠送信息")
+        if prop_give.act_id != act_id:
+            return self.reponse_json_error("Error", "不存在赠送信息")
+        if prop_give.give_status == 1:
+            return self.reponse_json_error("Error", "已被领取")
+        if prop_give.give_status == 2:
+            return self.reponse_json_error("Error", "已失效,无法领取")
+
+        user_detail = user_detail_model.get_entity("open_id=%s and act_id=%s", params=[open_id, act_id])
+        history_value = 0
+        update_sql = ""
+        if not user_detail:
+            return self.reponse_json_error("Error", "对不起，用户不存在")
+        if prop_give.prop_type == 3:
+            history_value = user_detail.tips_card_count
+        elif prop_give.prop_type == 4:
+            history_value = user_detail.redraw_card_count
+        else:
+            history_value = user_detail.perspective_card_count
+        prop_log = PropLog()
+        prop_log.app_id = app_id
+        prop_log.act_id = act_id
+        prop_log.open_id = open_id
+        prop_log.user_nick = user_info.user_nick
+        prop_log.change_type = 4
+        prop_log.operate_type = 0
+        prop_log.prop_type = prop_give.prop_type
+        prop_log.operate_value = 1
+        prop_log.history_value = history_value
+        prop_log.title = "领取赠送"
+        prop_log.create_date_int = SevenHelper.get_now_day_int()
+        prop_log.create_date = self.get_now_datetime()
+        info = {}
+        prop_log.remark = info
+        try:
+            db_transaction.begin_transaction()
+            prop_log_model.add_entity(prop_log)
+            prop_give_model.update_table("draw_open_id=%s,draw_user_nick=%s,give_status=1,modify_date=%s", params=[user_info.open_id, user_info.user_nick, self.get_now_datetime()])
+            user_detail_model.update_table(update_sql, "open_id=%s and act_id=%s", params=[open_id, act_id])
+            db_transaction.commit_transaction()
+
+        except Exception as ex:
+            db_transaction.rollback_transaction()
+            self.logging_link_error("GetPropGiveHandler:" + str(ex))
+            return self.reponse_json_error("Error", "系统繁忙,请稍后再试")
+        return self.reponse_json_success({"prop_type": prop_give.prop_type, "num": 1})
